function svm = simsvm_socp(S, y, eta, gamma, regopt, param)
% SIMSVM_SOCP  Solve the SOCP of SimSVM.
%   SVM = SIMSVM_SOCP(S, Y, ETA, GAMMA, REGOPT) calls MOSEK to solve
%   the primal problem of SimSVM, which is a convex conic program.
%
%   Problem:
%     minimize    1' xi / n + 2 eta 1' t + gamma h(a)
%     subject to  diag(y)(U z + b 1) >= 1 - xi,
%                 xi >= 0,
%                 t >= 0,
%                 lambda_i a_i >= 0,  i = 1, ..., n,
%                 z_i^2 <= 2 (lambda_i a_i) t_i,  i = 1, ..., n.
%
%   Input arguments:
%     S        similarity matrix
%     y        vector of class labels
%     eta      regularization parameter for the discriminant function
%     gamma    regularization parameter for the spectrum modifier
%     regopt   'a_l1'        h(a) = || a ||_1
%              'a_l2'        h(a) = || a ||_2
%              'a_linf'      h(a) = || a ||_inf
%              'aclip_l1'    h(a) = || a - a_clip ||_1
%              'aclip_l2'    h(a) = || a - a_clip ||_2
%              'aclip_linf'  h(a) = || a - a_clip ||_inf
%              'aflip_l1'    h(a) = || a - a_flip ||_1
%              'aflip_l2'    h(a) = || a - a_flip ||_2
%              'aflip_linf'  h(a) = || a - a_flip ||_inf
%              'lambda'      h(a) = || lambda (a - 1) ||_2
%     param    structure containing extra options
%     param.silence  'yes'  turn off MOSEK screen output
%                    'no'   turn on MOSEK screen output
%     param.detail   'yes'  save optimization details
%                    'no'   do not save optimization details
%
%   Output arguments:
%     svm      structure containing SimSVM parameters
%
%   Author: Yihua (James) Chen (email: yhchen@u.washington.edu)
%   Last modified: Jun-22-2010
%
%
% load Protein
% L=2*(mod(Label,2)-0.5)
% svm = simsvm_socp(Data, L, 1e-5, 2, 'a_l1',struct('silence','no','detail','yes'))
% [e,v] = eig(0.5*(svm.K+svm.K'))
% bar(diag(v))
%%
if nargin < 5 || nargin > 6
	error('Incorrect number of input arguments');
elseif nargin == 5
	param.silence = 'yes';
	param.detail = 'no';
else % nargin == 6
	if ~isfield(param, 'silence')
		param.silence = 'yes';
	end
	if ~isfield(param, 'detail')
		param.detail = 'no';
	end
end

n = length(S);
S = (S + S') / 2;
[V, D] = eig(S); % S = V * D * V'
lambda = real(diag(D));

a_clip = lambda > 0;
a_flip = sign(lambda);

epsilon = 1e-6;
tolerance = 1e-6;

idx = find(abs(lambda) > epsilon);
m = length(idx); % m <= n

U = V(:, idx);
lambda_sub = lambda(idx);

if strcmpi(param.silence, 'yes')
	mosekcmd = 'minimize echo(0)';
else
	mosekcmd = 'minimize';
end
errmsg = 'MOSEK encounters an error when solving the optimization problem';

prob = [];

%% Solve the SOCP of SimSVM with different regularizers
switch lower(regopt)
	case 'a_l1'
        %% h(a) = || a ||_1
		
		% Set up the parameters of the conic program
		prob.c = [zeros(m + 1, 1); ones(n, 1) / n; ...
			gamma ./ abs(lambda_sub); 2 * eta * ones(m, 1)];
        A = [diag(y) * U, y, eye(n), zeros(n, 2 * m)];
        A(abs(A) < epsilon) = 0;
        prob.a = sparse(A);
		prob.blc = ones(n, 1);
		prob.buc = [];
		prob.blx = sparse([-inf * ones(m + 1, 1); zeros(2 * m + n, 1)]);
		prob.bux = [];
		prob.cones = cell(m, 1);
		for i = 1:m
			prob.cones{i}.type = 'MSK_CT_RQUAD'; % Roated quadratic cone
			prob.cones{i}.sub = [m + n + 1 + i, 2 * m + n + 1 + i, i];
		end
		
		% Solve the conic program by MOSEK
        if strcmpi(param.detail, 'yes')
            tic;
            [rcode, res] = mosekopt(mosekcmd, prob);
            svm.time = toc;
        else
            [rcode, res] = mosekopt(mosekcmd, prob);
        end
        if rcode ~= 0 && rcode ~= 4006
			error(errmsg);
        end
        
        % Get the SimSVM parameters
        z = zeros(n, 1);
        z(idx) = res.sol.itr.xx(1:m);
        svm.b = res.sol.itr.xx(m + 1);
        svm.a = zeros(n, 1);
        svm.a(idx) = res.sol.itr.xx((m + n + 2):(2 * m + n + 1)) ./ ...
            lambda_sub;
        svm.a(abs(svm.a) < epsilon) = 0;
        svm.c = V * pinv(diag(svm.a .* lambda), tolerance) * z;

	case 'a_l2'
        %% h(a) = || a ||_2
        
        % Set up the parameters of the conic program
        prob.c = [zeros(m + 1, 1); ones(n, 1) / n; zeros(m, 1); ...
            2 * eta * ones(m, 1); gamma; zeros(m, 1)];
        A = [diag(y) * U, y, eye(n), zeros(n, 3 * m + 1); ...
            zeros(m, m + n + 1), eye(m), zeros(m, m + 1), ...
            -diag(abs(lambda_sub))];
        A(abs(A) < epsilon) = 0;
        prob.a = sparse(A);
        prob.blc = [ones(n, 1); zeros(m, 1)];
        prob.buc = [inf * ones(n, 1); zeros(m, 1)];
        prob.blx = sparse([-inf * ones(m + 1, 1); zeros(3 * m + n + 1, 1)]);
        prob.bux = [];
        prob.cones = cell(m + 1, 1);
        for i = 1:m
            prob.cones{i}.type = 'MSK_CT_RQUAD'; % Rotated quadratic cone
            prob.cones{i}.sub = [m + n + 1 + i, 2 * m + n + 1 + i, i];
        end
        prob.cones{m + 1}.type = 'MSK_CT_QUAD'; % Quadratic cone
        prob.cones{m + 1}.sub = 3 * m + n + 1 + (1:(m + 1));
        
        % Solve the conic program by MOSEK
        if strcmpi(param.detail, 'yes')
            tic;
            [rcode, res] = mosekopt(mosekcmd, prob);
            svm.time = toc;
        else
            [rcode, res] = mosekopt(mosekcmd, prob);
        end
		if rcode ~= 0 && rcode ~= 4006
			error(errmsg);
		end
        
        % Get the SimSVM parameters
        z = zeros(n, 1);
        z(idx) = res.sol.itr.xx(1:m);
        svm.b = res.sol.itr.xx(m + 1);
        svm.a = zeros(n, 1);
        svm.a(idx) = sign(lambda_sub) .* ...
            res.sol.itr.xx(3 * m + n + 2 + (1:m));
        svm.a(abs(svm.a) < epsilon) = 0;
        svm.c = V * pinv(diag(svm.a .* lambda), tolerance) * z;

	case 'a_linf'
        %% h(a) = || a ||_inf
        
        % Set up the parameters of the conic program
        prob.c = [zeros(m + 1, 1); ones(n, 1) / n; zeros(m, 1); ...
            2 * eta * ones(m, 1); gamma];
        A = [diag(y) * U, y, eye(n), zeros(n, 2 * m + 1); ...
            zeros(m, m + n + 1), -eye(m), zeros(m), abs(lambda_sub)];
        A(abs(A) < epsilon) = 0;
        prob.a = sparse(A);
        prob.blc = [ones(n, 1); zeros(m, 1)];
        prob.buc = [];
        prob.blx = sparse([-inf * ones(m + 1, 1); zeros(2 * m + n + 1, 1)]);
        prob.bux = [];
        prob.cones = cell(m, 1);
        for i = 1:m
            prob.cones{i}.type = 'MSK_CT_RQUAD'; % Rotated quadratic cone
            prob.cones{i}.sub = [m + n + 1 + i, 2 * m + n + 1 + i, i];
        end
        
        % Solve the conic program by MOSEK
        if strcmpi(param.detail, 'yes')
            tic;
            [rcode, res] = mosekopt(mosekcmd, prob);
            svm.time = toc;
        else
            [rcode, res] = mosekopt(mosekcmd, prob);
        end
		if rcode ~= 0 && rcode ~= 4006
			error(errmsg);
		end
        
        % Get the SimSVM parameters
        z = zeros(n, 1);
        z(idx) = res.sol.itr.xx(1:m);
        svm.b = res.sol.itr.xx(m + 1);
        svm.a = zeros(n, 1);
        svm.a(idx) = res.sol.itr.xx((m + n + 2):(2 * m + n + 1)) ./ ...
            lambda_sub;
        svm.a(abs(svm.a) < epsilon) = 0;
        svm.c = V * pinv(diag(svm.a .* lambda), tolerance) * z;

    case 'aclip_l1'
        %% h(a) = || a - a_clip ||_1
        
		% Set up the parameters of the conic program
        prob.c = [zeros(m + 1, 1); ones(n, 1) / n; zeros(m, 1); ...
            2 * eta * ones(m, 1); gamma * ones(m, 1); zeros(m, 1)];
        A = [diag(y) * U, y, eye(n), zeros(n, 4 * m); ...
            zeros(m, m + n + 1), eye(m), zeros(m, 2 *m), ...
            -diag(lambda_sub); zeros(m, 3 * m + n + 1), eye(m), eye(m); ...
            zeros(m, 3 * m + n + 1), eye(m), -eye(m)];
        A(abs(A) < epsilon) = 0;
        prob.a = sparse(A);
        prob.blc = [ones(n, 1); a_clip(idx) .* lambda_sub; ...
            zeros(2 * m, 1)];
        prob.buc = [inf * ones(n, 1); a_clip(idx) .* lambda_sub; ...
            inf * ones(2 * m, 1)];
        prob.blx = [-inf * ones(m + 1, 1); zeros(3 * m + n, 1); ...
            -inf * ones(m, 1)];
        prob.bux = [];
        prob.cones = cell(m, 1);
        for i = 1:m
            prob.cones{i}.type = 'MSK_CT_RQUAD'; % Rotated quadratic cone
            prob.cones{i}.sub = [m + n + 1 + i, 2 * m + n + 1 + i, i];
        end
		
		% Solve the conic program by MOSEK
        if strcmpi(param.detail, 'yes')
            tic;
            [rcode, res] = mosekopt(mosekcmd, prob);
            svm.time = toc;
        else
            [rcode, res] = mosekopt(mosekcmd, prob);
        end
        if rcode ~= 0 && rcode ~= 4006
            error(errmsg);
        end
        
		% Get the SimSVM parameters
        z = zeros(n, 1);
        z(idx) = res.sol.itr.xx(1:m);
        svm.b = res.sol.itr.xx(m + 1);
        svm.a = zeros(n, 1);
        svm.a(idx) = res.sol.itr.xx((m + n + 2):(2 * m + n + 1)) ./ ...
            lambda_sub;
        svm.a(abs(svm.a) < epsilon) = 0;
        svm.c = V * pinv(diag(svm.a .* lambda), tolerance) * z;

    case 'aclip_l2'
        %% h(a) = || a - a_clip ||_2
		
		% Set up the parameters of the conic program
		prob.c = [zeros(m + 1, 1); ones(n, 1) / n; zeros(m, 1); ...
			2 * eta * ones(m, 1); gamma; zeros(m, 1)];
        A = [diag(y) * U, y, eye(n), zeros(n, 3 * m + 1); ...
			zeros(m, m + n + 1), eye(m), zeros(m, m + 1), ...
			-diag(lambda_sub)];
        A(abs(A) < epsilon) = 0;
		prob.a = sparse(A);
		prob.blc = [ones(n, 1); a_clip(idx) .* lambda_sub];
		prob.buc = [inf * ones(n, 1); a_clip(idx) .* lambda_sub];
		prob.blx = [-inf * ones(m + 1, 1); zeros(2 * m + n + 1, 1); ...
			-inf * ones(m, 1)];
		prob.bux = [];
		prob.cones = cell(m + 1, 1);
        for i = 1:m
            prob.cones{i}.type = 'MSK_CT_RQUAD'; % Rotated quadratic cone
            prob.cones{i}.sub = [m + n + 1 + i, 2 * m + n + 1 + i, i];
        end
        prob.cones{m + 1}.type = 'MSK_CT_QUAD'; % Quadratic cone
        prob.cones{m + 1}.sub = 3 * m + n + 1 + (1:(m + 1));
		
		% Solve the conic program by MOSEK
        if strcmpi(param.detail, 'yes')
            tic;
            [rcode, res] = mosekopt(mosekcmd, prob);
            svm.time = toc;
        else
            [rcode, res] = mosekopt(mosekcmd, prob);
        end
		if rcode ~= 0 && rcode ~= 4006
			error(errmsg);
		end
		
		% Get the SimSVM parameters
		z = zeros(n, 1);
        z(idx) = res.sol.itr.xx(1:m);
        svm.b = res.sol.itr.xx(m + 1);
		svm.a = zeros(n, 1);
        svm.a(idx) = res.sol.itr.xx((m + n + 2):(2 * m + n + 1)) ./ ...
            lambda_sub;
        svm.a(abs(svm.a) < epsilon) = 0;
        svm.c = V * pinv(diag(svm.a .* lambda), tolerance) * z;

    case 'aclip_linf'
        %% h(a) = || a - a_clip ||_inf
        
        % Set up the parameters of the conic program
        prob.c = [zeros(m + 1, 1); ones(n, 1) / n; zeros(m, 1); ...
            2 * eta * ones(m, 1); gamma; zeros(m, 1)];
        A = [diag(y) * U, y, eye(n), zeros(n, 3 * m + 1); ...
            zeros(m, m + n + 1), eye(m), zeros(m, m + 1), ...
            -diag(lambda_sub); zeros(m, 3 * m + n + 1), ones(m, 1), ...
            eye(m); zeros(m, 3 * m + n + 1), ones(m, 1), -eye(m)];
        A(abs(A) < epsilon) = 0;
        prob.a = sparse(A);
        prob.blc = [ones(n, 1); a_clip(idx) .* lambda_sub; ...
            zeros(2 * m, 1)];
        prob.buc = [inf * ones(n, 1); a_clip(idx) .* lambda_sub; ...
            inf * ones(2 * m, 1)];
        prob.blx = [-inf * ones(m + 1, 1); zeros(2 * m + n + 1, 1); ...
            -inf * ones(m, 1)];
        prob.bux = [];
        prob.cones = cell(m, 1);
        for i = 1:m
            prob.cones{i}.type = 'MSK_CT_RQUAD'; % Rotated quadratic cone
            prob.cones{i}.sub = [m + n + 1 + i, 2 * m + n + 1 + i, i];
        end
        
        % Solve the conic program by MOSEK
        if strcmpi(param.detail, 'yes')
            tic;
            [rcode, res] = mosekopt(mosekcmd, prob);
            svm.time = toc;
        else
            [rcode, res] = mosekopt(mosekcmd, prob);
        end
		if rcode ~= 0 && rcode ~= 4006
			error(errmsg);
		end
        
        % Get the SimSVM parameters
        z = zeros(n, 1);
        z(idx) = res.sol.itr.xx(1:m);
        svm.b = res.sol.itr.xx(m + 1);
        svm.a = zeros(n, 1);
        svm.a(idx) = res.sol.itr.xx((m + n + 2):(2 * m + n + 1)) ./ ...
            lambda_sub;
        svm.a(abs(svm.a) < epsilon) = 0;
        svm.c = V * pinv(diag(svm.a .* lambda), tolerance) * z;
        
    case 'aflip_l1'
        %% h(a) = || a - a_flip ||_1
		
		% Set up the parameters of the conic program
        prob.c = [zeros(m + 1, 1); ones(n, 1) / n; zeros(m, 1); ...
            2 * eta * ones(m, 1); gamma * ones(m, 1); zeros(m, 1)];
        A = [diag(y) * U, y, eye(n), zeros(n, 4 * m); ...
            zeros(m, m + n + 1), eye(m), zeros(m, 2 *m), ...
            -diag(lambda_sub); zeros(m, 3 * m + n + 1), eye(m), eye(m); ...
            zeros(m, 3 * m + n + 1), eye(m), -eye(m)];
        A(abs(A) < epsilon) = 0;
        prob.a = sparse(A);
        prob.blc = [ones(n, 1); a_flip(idx) .* lambda_sub; ...
            zeros(2 * m, 1)];
        prob.buc = [inf * ones(n, 1); a_flip(idx) .* lambda_sub; ...
            inf * ones(2 * m, 1)];
        prob.blx = [-inf * ones(m + 1, 1); zeros(3 * m + n, 1); ...
            -inf * ones(m, 1)];
        prob.bux = [];
        prob.cones = cell(m, 1);
        for i = 1:m
            prob.cones{i}.type = 'MSK_CT_RQUAD'; % Rotated quadratic cone
            prob.cones{i}.sub = [m + n + 1 + i, 2 * m + n + 1 + i, i];
        end
		
		% Solve the conic program by MOSEK
        if strcmpi(param.detail, 'yes')
            tic;
            [rcode, res] = mosekopt(mosekcmd, prob);
            svm.time = toc;
        else
            [rcode, res] = mosekopt(mosekcmd, prob);
        end
        if rcode ~= 0 && rcode ~= 4006
            error(errmsg);
        end
        
		% Get the SimSVM parameters
        z = zeros(n, 1);
        z(idx) = res.sol.itr.xx(1:m);
        svm.b = res.sol.itr.xx(m + 1);
        svm.a = zeros(n, 1);
        svm.a(idx) = res.sol.itr.xx((m + n + 2):(2 * m + n + 1)) ./ ...
            lambda_sub;
        svm.a(abs(svm.a) < epsilon) = 0;
        svm.c = V * pinv(diag(svm.a .* lambda), tolerance) * z;
        
    case 'aflip_l2'
        %% h(a) = || a - a_flip ||_2
		
		% Set up the parameters of the conic program
		prob.c = [zeros(m + 1, 1); ones(n, 1) / n; zeros(m, 1); ...
			2 * eta * ones(m, 1); gamma; zeros(m, 1)];
        A = [diag(y) * U, y, eye(n), zeros(n, 3 * m + 1); ...
			zeros(m, m + n + 1), eye(m), zeros(m, m + 1), ...
			-diag(lambda_sub)];
        A(abs(A) < epsilon) = 0;
		prob.a = sparse(A);
		prob.blc = [ones(n, 1); a_flip(idx) .* lambda_sub];
		prob.buc = [inf * ones(n, 1); a_flip(idx) .* lambda_sub];
		prob.blx = [-inf * ones(m + 1, 1); zeros(2 * m + n + 1, 1); ...
			-inf * ones(m, 1)];
		prob.bux = [];
		prob.cones = cell(m + 1, 1);
        for i = 1:m
            prob.cones{i}.type = 'MSK_CT_RQUAD'; % Rotated quadratic cone
            prob.cones{i}.sub = [m + n + 1 + i, 2 * m + n + 1 + i, i];
        end
        prob.cones{m + 1}.type = 'MSK_CT_QUAD'; % Quadratic cone
        prob.cones{m + 1}.sub = 3 * m + n + 1 + (1:(m + 1));
		
		% Solve the conic program by MOSEK
        if strcmpi(param.detail, 'yes')
            tic;
            [rcode, res] = mosekopt(mosekcmd, prob);
            svm.time = toc;
        else
            [rcode, res] = mosekopt(mosekcmd, prob);
        end
		if rcode ~= 0 && rcode ~= 4006
			error(errmsg);
		end
		
		% Get the SimSVM parameters
		z = zeros(n, 1);
        z(idx) = res.sol.itr.xx(1:m);
        svm.b = res.sol.itr.xx(m + 1);
		svm.a = zeros(n, 1);
        svm.a(idx) = res.sol.itr.xx((m + n + 2):(2 * m + n + 1)) ./ ...
            lambda_sub;
        svm.a(abs(svm.a) < epsilon) = 0;
        svm.c = V * pinv(diag(svm.a .* lambda), tolerance) * z;
        
    case 'aflip_linf'
        %% h(a) = || a - a_flip ||_inf
		
		% Set up the parameters of the conic program
        prob.c = [zeros(m + 1, 1); ones(n, 1) / n; zeros(m, 1); ...
            2 * eta * ones(m, 1); gamma; zeros(m, 1)];
        A = [diag(y) * U, y, eye(n), zeros(n, 3 * m + 1); ...
            zeros(m, m + n + 1), eye(m), zeros(m, m + 1), ...
            -diag(lambda_sub); zeros(m, 3 * m + n + 1), ones(m, 1), ...
            eye(m); zeros(m, 3 * m + n + 1), ones(m, 1), -eye(m)];
        A(abs(A) < epsilon) = 0;
        prob.a = sparse(A);
        prob.blc = [ones(n, 1); a_flip(idx) .* lambda_sub; ...
            zeros(2 * m, 1)];
        prob.buc = [inf * ones(n, 1); a_flip(idx) .* lambda_sub; ...
            inf * ones(2 * m, 1)];
        prob.blx = [-inf * ones(m + 1, 1); zeros(2 * m + n + 1, 1); ...
            -inf * ones(m, 1)];
        prob.bux = [];
        prob.cones = cell(m, 1);
        for i = 1:m
            prob.cones{i}.type = 'MSK_CT_RQUAD'; % Rotated quadratic cone
            prob.cones{i}.sub = [m + n + 1 + i, 2 * m + n + 1 + i, i];
        end
        
        % Solve the conic program by MOSEK
        if strcmpi(param.detail, 'yes')
            tic;
            [rcode, res] = mosekopt(mosekcmd, prob);
            svm.time = toc;
        else
            [rcode, res] = mosekopt(mosekcmd, prob);
        end
		if rcode ~= 0 && rcode ~= 4006
			error(errmsg);
		end
        
        % Get the SimSVM parameters
        z = zeros(n, 1);
        z(idx) = res.sol.itr.xx(1:m);
        svm.b = res.sol.itr.xx(m + 1);
        svm.a = zeros(n, 1);
        svm.a(idx) = res.sol.itr.xx((m + n + 2):(2 * m + n + 1)) ./ ...
            lambda_sub;
        svm.a(abs(svm.a) < epsilon) = 0;
        svm.c = V * pinv(diag(svm.a .* lambda), tolerance) * z;
        
    case 'lambda'
        %% h(a) = || lambda (a - 1) ||_2
        
        % Set up the parameters of the conic program
        prob.c = [zeros(m + 1, 1); ones(n, 1) / n; zeros(m, 1); ...
			2 * eta * ones(m, 1); gamma; zeros(m, 1)];
        A = [diag(y) * U, y, eye(n), zeros(n, 3 * m + 1); ...
			zeros(m, m + n + 1), eye(m), zeros(m, m + 1), -eye(m)];
        A(abs(A) < epsilon) = 0;
		prob.a = sparse(A);
		prob.blc = [ones(n, 1); lambda_sub];
		prob.buc = [inf * ones(n, 1); lambda_sub];
		prob.blx = [-inf * ones(m + 1, 1); zeros(2 * m + n + 1, 1); ...
			-inf * ones(m, 1)];
		prob.bux = [];
		prob.cones = cell(m + 1, 1);
        for i = 1:m
            prob.cones{i}.type = 'MSK_CT_RQUAD'; % Rotated quadratic cone
            prob.cones{i}.sub = [m + n + 1 + i, 2 * m + n + 1 + i, i];
        end
        prob.cones{m + 1}.type = 'MSK_CT_QUAD'; % Quadratic cone
        prob.cones{m + 1}.sub = 3 * m + n + 1 + (1:(m + 1));
        
        % Solve the conic program by MOSEK
        if strcmpi(param.detail, 'yes')
            tic;
            [rcode, res] = mosekopt(mosekcmd, prob);
            svm.time = toc;
        else
            [rcode, res] = mosekopt(mosekcmd, prob);
        end
		if rcode ~= 0 && rcode ~= 4006
			error(errmsg);
		end
        
        % Get the SimSVM parameters
		z = zeros(n, 1);
        z(idx) = res.sol.itr.xx(1:m);
        svm.b = res.sol.itr.xx(m + 1);
		svm.a = zeros(n, 1);
        svm.a(idx) = res.sol.itr.xx((m + n + 2):(2 * m + n + 1)) ./ ...
            lambda_sub;
        svm.a(abs(svm.a) < epsilon) = 0;
        svm.c = V * pinv(diag(svm.a .* lambda), tolerance) * z;
        
    otherwise
        %% Unknown regularizer
		error('Unknown regularization option');
end

%%
svm.A = V * diag(svm.a) * V';

if strcmpi(param.detail, 'yes')
    K = svm.A * S;
    tmp1 = y .* (K * svm.c + svm.b);
    tmp2 = 1 - tmp1;
    svm.K      = K;
    svm.emperr = sum(tmp1 < 0) / n;
    svm.emprisk = sum( tmp2( tmp2 > 0 ) ) / n;
    svm.objval = res.sol.itr.pobjval;
end